@@ -1,11 +1,11 @@
 #![allow(non_snake_case)]
-use from_bytes::*;
-use from_bytes_derive::*;
-use packed_struct::prelude::*;
 use chrono::{DateTime, Utc};
 use std::io::{Read, Seek, SeekFrom};
 use winstructs::timestamp::WinTimestamp;
 use winstructs::ntfs::mft_reference::MftReference;
+use binread::prelude::*;
+use binread::derive_binread;
+use binread::ReadOptions;
 
 use crate::usn_reader_error::*;
 use crate::usn_reason::*;
@@ -16,35 +16,23 @@ pub struct CommonUsnRecord {
   pub data: UsnRecordData,
 }
 
-fn utf16_from_slice<RS>(slice: &mut RS, offset: usize, characters: usize) -> Result<String, UsnReaderError> where RS: Read + Seek {
-  let mut name_bytes = vec![0; characters * 2];
-
-  slice.seek(SeekFrom::Start(offset as u64))?;
-  slice.read_exact(&mut name_bytes)?;
-
-  let mut name_chars = Vec::new();
-  for idx in 0..characters {
-    name_chars.push(name_bytes[2*idx] as u16 | ((name_bytes[2*idx + 1] as u16) << 8 as u8));
-  }
-  Ok(String::from_utf16_lossy(&name_chars[..]))
-}
-
 impl CommonUsnRecord {
   pub fn from<RS>(data: &mut RS, index: &mut usize) -> std::result::Result<Self, UsnReaderError> where RS: Read + Seek {
     data.seek(SeekFrom::Start(*index as u64))?;
-    let mut header = *UsnRecordCommonHeader::from_stream(data)?;
+    let mut header: UsnRecordCommonHeader = data.read_le()?;
 
     while header.RecordLength == 0 {
       /* looks like a cluster change, round index up to the next cluster */
       *index += 0x1000 - (*index & 0xfff);
+      log::debug!("found end of cluster, seeking to {}", index);
 
       /* reread header at new address */
       data.seek(SeekFrom::Start(*index as u64))?;
-      header = *UsnRecordCommonHeader::from_stream(data)?;
+      header = data.read_le()?;
     }
 
     let usn_data = match header.MajorVersion {
-      2 => UsnRecordData::V2(UsnRecordV2::from(data, *index)?),
+      2 => UsnRecordData::V2(UsnRecordV2::from(data)?),
       3 => {
         return Err(UsnReaderError::SyntaxError(format!(
           "Version 3 records (ReFS only) are not supported yes"
@@ -105,9 +93,18 @@ impl UsnRecordData {
     }
   }
 }
+pub struct CurPos(pub u64);
 
-#[derive(PackedStruct, Debug, StructFromBytes, PackedSize)]
-#[packed_struct(bit_numbering = "msb0", endian = "lsb")]
+impl BinRead for CurPos {
+    type Args = ();
+
+    fn read_options<R: Read + Seek>(reader: &mut R, _ro: &ReadOptions, _args: Self::Args) -> BinResult<Self> {
+        Ok(CurPos(reader.seek(SeekFrom::Current(0))?))
+    }
+}
+
+#[derive(BinRead, Debug)]
+#[br(little)]
 pub struct UsnRecordCommonHeader {
   /// The total length of a record, in bytes.
   ///
@@ -141,20 +138,54 @@ pub struct UsnRecordCommonHeader {
 /// which is common through USN_RECORD_V2, USN_RECORD_V3 and USN_RECORD_V4.
 ///
 /// https://docs.microsoft.com/de-de/windows/win32/api/winioctl/ns-winioctl-usn_record_common_header
-#[derive(PackedStruct, Debug, StructFromBytes, PackedSize)]
-#[packed_struct(bit_numbering = "msb0", endian = "lsb")]
+
+#[derive_binread]
+#[br(little)]
 pub struct BinaryUsnRecordV2 {
+
+  /// the following field is not really part of UsnRecordV2
+  #[br(temp)]
+  pub StartingPosition: CurPos,
+
   pub FileReferenceNumber: u64,
   pub ParentFileReferenceNumber: u64,
   pub Usn: i64,
   pub TimeStamp: [u8;8],
   pub Reason: u32,
   pub SourceInfo: u32,
+
+  /// The unique security identifier assigned to the file or directory
+  /// associated with this record.
   pub SecurityId: u32,
+
+  /// The attributes for the file or directory associated with this record, as
+  /// returned by the GetFileAttributes function. Attributes of streams
+  /// associated with the file or directory are excluded.
   pub FileAttributes: u32,
+
+  /// The length of the name of the file or directory associated with this
+  /// record, in bytes. The FileName member contains this name. Use this member
+  /// to determine file name length, rather than depending on a trailing '\0'
+  /// to delimit the file name in FileName.
+  #[br(temp, little)]
   pub FileNameLength: u16,
+
+  /// The offset of the FileName member from the beginning of the structure.
   pub FileNameOffset: u16,
-  pub FileName: u16,
+
+  /// The name of the file or directory associated with this record in Unicode
+  /// format. This file or directory name is of variable length.
+  ///
+  /// When working with FileName, do not count on the file name that contains
+  /// a trailing '\0' delimiter, but instead determine the length of the file
+  /// name by using FileNameLength.
+  ///
+  /// Do not perform any compile-time pointer arithmetic using FileName.
+  /// Instead, make necessary calculations at run time by using the value of
+  /// the FileNameOffset member. Doing so helps make your code compatible with
+  /// any future versions of USN_RECORD_V2.
+  #[br(offset=StartingPosition.0 + (FileNameOffset as u64), count=FileNameLength/2)]
+  pub FileName: Vec<u16>,
 }
 
 #[derive(Debug)]
@@ -171,11 +202,10 @@ pub struct UsnRecordV2 {
 }
 
 impl UsnRecordV2 {
-  fn from<RS> (data: &mut RS, record_index: usize) -> std::result::Result<Self, UsnReaderError> where RS: Read + Seek {
-    data.seek(SeekFrom::Start((record_index + UsnRecordCommonHeader::packed_size()) as u64))?;
-    let record = BinaryUsnRecordV2::from_stream(data)?;
+  fn from<RS> (data: &mut RS) -> std::result::Result<Self, UsnReaderError> where RS: Read + Seek {
+    let record: BinaryUsnRecordV2 = data.read_le()?;
 
-    let filename = utf16_from_slice(data, record_index + record.FileNameOffset as usize, (record.FileNameLength / 2) as usize)?;
+    let filename = String::from_utf16_lossy(&record.FileName);
 
     let file_reference = MftReference::from(record.FileReferenceNumber);
     let parent_reference = MftReference::from(record.ParentFileReferenceNumber);
@@ -197,8 +227,8 @@ impl UsnRecordV2 {
 }
 
 /*
-#[derive(PackedStruct, Debug, StructFromBytes, PackedSize)]
-#[packed_struct(bit_numbering = "msb0", endian = "lsb")]
+#[derive(BinRead)]
+#[br(little)]
 pub struct UsnRecordV3 {
   pub FileReferenceNumber: [u8; 16],
   pub ParentFileReferenceNumber: [u8; 16],
@@ -215,8 +245,8 @@ pub struct UsnRecordV3 {
 }
 */
 /*
-#[derive(PackedStruct, Debug, StructFromBytes, PackedSize)]
-#[packed_struct(bit_numbering = "msb0", endian = "lsb")]
+#[derive(BinRead)]
+#[br(little)]
 pub struct UsnRecordV4 {
   pub FileReferenceNumber: [u8; 16],
   pub ParentFileReferenceNumber: [u8; 16],
@@ -229,8 +259,8 @@ pub struct UsnRecordV4 {
   pub ExtentSize: u16,
 }
 
-#[derive(PackedStruct, Debug, StructFromBytes, PackedSize)]
-#[packed_struct(bit_numbering = "msb0", endian = "lsb")]
+#[derive(BinRead)]
+#[br(little)]
 pub struct UsnRecordExtend {
   pub Offset: u64,
   pub Length: u64,
